@@ -25,11 +25,6 @@ import openpaperwork_core.promise
 from .... import (_, sync)
 
 
-# Beware that we use Sqlite, but sqlite python module is not thread-safe
-# --> all the calls to sqlite module functions must happen on the main loop,
-# even those in the transactions (which are run in a thread)
-
-
 LOGGER = logging.getLogger(__name__)
 ID = "label_guesser"
 CREATE_TABLES = [
@@ -132,7 +127,10 @@ class UpdatableVectorizer(object):
         vocabulary = self.db_cursor.execute(
             "SELECT word, feature FROM vocabulary"
         )
-        vocabulary = {k: v for (k, v) in vocabulary}
+        vocabulary = {k.strip(): v for (k, v) in vocabulary}
+        if "" in vocabulary:
+            vocabulary.pop("")
+
         self.updatable_vocabulary = vocabulary
         self.last_feature_id = max(vocabulary.values(), default=-1)
 
@@ -155,9 +153,9 @@ class UpdatableVectorizer(object):
                     "  WHERE NOT EXISTS("
                     "   SELECT 1 FROM vocabulary WHERE word = ?"
                     "  )",
-                    (word, self.last_feature_id, word)
+                    (word.strip(), self.last_feature_id, word)
                 )
-                self.updatable_vocabulary[word] = self.last_feature_id
+                self.updatable_vocabulary[word.strip()] = self.last_feature_id
         LOGGER.info(
             "Vocabulary contains %d words after fitting", self.last_feature_id
         )
@@ -167,12 +165,16 @@ class UpdatableVectorizer(object):
     def transform(self, corpus):
         # IMPORTANT: we must use use_idf=False here because we want the values
         # in each feature vector to be independant from other vectors.
-        vectorizer = sklearn.feature_extraction.text.TfidfVectorizer(
-            use_idf=False, vocabulary=self.updatable_vocabulary
-        )
-        features = vectorizer.fit_transform(corpus)
-        LOGGER.info("%s features extracted", features.shape)
-        return features
+        try:
+            vectorizer = sklearn.feature_extraction.text.TfidfVectorizer(
+                use_idf=False, vocabulary=self.updatable_vocabulary
+            )
+            features = vectorizer.fit_transform(corpus)
+            LOGGER.info("%s features extracted", features.shape)
+            return features
+        except ValueError as exc:
+            LOGGER.warning("Failed to extract features", exc_info=exc)
+            return scipy.sparse.csr_matrix((0, 0))
 
     def _find_unused(self):
         doc_features = self.db_cursor.execute(
@@ -180,6 +182,8 @@ class UpdatableVectorizer(object):
         )
         sum_features = None
         for (doc_vector,) in doc_features:
+            if doc_vector is None:
+                continue
             required_padding = (
                 self.last_feature_id + 1 - doc_vector.shape[0]
             )
@@ -452,11 +456,15 @@ class Corpus(object):
                 (doc_id,)
             )
             for vector in vectors:
-                all_features[doc_id] = vector[0]
+                vector = vector[0]
+                if vector is None:
+                    continue
+                all_features[doc_id] = vector
 
         all_features = [
             [weight, doc_id, all_features[doc_id]]
             for (doc_id, weight) in doc_weights.items()
+            if doc_id in all_features
         ]
         all_features.sort(reverse=True)
         doc_ids = [
@@ -539,16 +547,15 @@ class LabelGuesserTransaction(sync.BaseTransaction):
             # (see add_doc() -> _set_guessed_labels())
             self.plugin.classifiers_cond.wait()
 
-        self.cursor = sqlite3.connect(
-            self.core.call_success("fs_unsafe", self.plugin.sql_file),
-            detect_types=sqlite3.PARSE_DECLTYPES
+        self.cursor = self.plugin.core.call_success(
+            "sqlite_open",
+            self.plugin.sql_url,
+            detect_types=sqlite3.PARSE_DECLTYPES,
         )
         self.cursor.execute("BEGIN TRANSACTION")
         self.vectorizer = UpdatableVectorizer(self.core, self.cursor)
 
     def add_doc(self, doc_id):
-        self._lazyinit_transaction()
-
         if self.guess_labels:
             # we have a higher priority than index plugins, so it is a good
             # time to update the document labels
@@ -565,8 +572,12 @@ class LabelGuesserTransaction(sync.BaseTransaction):
                 )
                 return
 
+            self._lazyinit_transaction()
             self.plugin._set_guessed_labels(doc_url)
 
+        else:
+            self._lazyinit_transaction()
+
         self.notify_progress(
             ID, _("Label guesser: added document %s") % doc_id
         )
@@ -625,7 +636,12 @@ class LabelGuesserTransaction(sync.BaseTransaction):
         self.core.call_all("doc_get_text_by_url", doc_txt, doc_url)
         doc_txt = "\n\n".join(doc_txt).strip()
 
-        vector = self.vectorizer.partial_fit_transform([doc_txt])[0].toarray()
+        vector = self.vectorizer.partial_fit_transform([doc_txt])
+        if vector.shape[0] <= 0 or vector.shape[1] <= 0:
+            vector = numpy.array([])
+        else:
+            vector = vector[0].toarray()
+
         self.cursor.execute(
             "INSERT INTO features (doc_id, vector) VALUES (?, ?)",
             (doc_id, vector)
@@ -639,7 +655,9 @@ class LabelGuesserTransaction(sync.BaseTransaction):
             )
             if self.cursor is not None:
                 self.cursor.execute("ROLLBACK")
-                self.cursor.close()
+                self.core.call_success(
+                    "sqlite_close", self.cursor, optimize=False
+                )
                 self.cursor = None
             self.notify_done(ID)
 
@@ -682,7 +700,7 @@ class LabelGuesserTransaction(sync.BaseTransaction):
             )
 
             self.cursor.execute("COMMIT")
-            self.cursor.close()
+            self.core.call_success("sqlite_close", self.cursor)
 
             self.notify_done(ID)
             self.core.call_success(
@@ -728,7 +746,7 @@ class Plugin(openpaperwork_core.PluginBase):
         self.classifiers_cond = threading.Condition(threading.Lock())
 
         self.bayes_dir = None
-        self.sql_file = None
+        self.sql_url = None
 
         SqliteNumpyArrayHandler.register()
 
@@ -743,6 +761,10 @@ class Plugin(openpaperwork_core.PluginBase):
                 'interface': 'config',
                 'defaults': ['openpaperwork_core.config'],
             },
+            {
+                'interface': 'data_dir_handler',
+                'defaults': ['paperwork_backend.datadirhandler'],
+            },
             {
                 'interface': 'data_versioning',
                 'defaults': ['openpaperwork_core.data_versioning'],
@@ -768,8 +790,8 @@ class Plugin(openpaperwork_core.PluginBase):
                 'defaults': ['openpaperwork_gtk.mainloop.glib'],
             },
             {
-                'interface': 'data_dir_handler',
-                'defaults': ['paperwork_backend.datadirhandler'],
+                'interface': 'sqlite',
+                'defaults': ['openpaperwork_core.sqlite'],
             },
             {
                 'interface': 'transaction_manager',
@@ -804,12 +826,17 @@ class Plugin(openpaperwork_core.PluginBase):
 
         self.core.call_success("fs_mkdir_p", self.bayes_dir)
 
-        self.sql_file = self.core.call_success(
+        self.sql_url = self.core.call_success(
             "fs_join", self.bayes_dir, 'label_guesser.db'
         )
-        self.sql = sqlite3.connect(
-            self.core.call_success("fs_unsafe", self.sql_file),
-            detect_types=sqlite3.PARSE_DECLTYPES
+        LOGGER.info("Opening guesswork.label.sklearn database")
+        self.sql = self.core.call_one(
+            "sqlite_execute",
+            self.core.call_success,
+            "sqlite_open",
+            self.sql_url,
+            detect_types=sqlite3.PARSE_DECLTYPES,
+            timeout=60
         )
         for query in CREATE_TABLES:
             self.sql.execute(query)
@@ -826,9 +853,17 @@ class Plugin(openpaperwork_core.PluginBase):
         )
 
     def on_data_dir_changed(self):
-        self.sql.close()
+        self.on_quit()
         self._init()
 
+    def on_quit(self):
+        LOGGER.info("Closing guesswork.label.sklearn database")
+        self.core.call_one(
+            "sqlite_execute",
+            self.core.call_success,
+            "sqlite_close", self.sql
+        )
+
     def reload_label_guessers(self):
         self.classifiers = None
         self.classifiers_loaded = True
@@ -843,9 +878,10 @@ class Plugin(openpaperwork_core.PluginBase):
     def _reload_label_guessers(self):
         with self.classifiers_cond:
             try:
-                cursor = sqlite3.connect(
-                    self.core.call_success("fs_unsafe", self.sql_file),
-                    detect_types=sqlite3.PARSE_DECLTYPES
+                cursor = self.core.call_one(
+                    "sqlite_open",
+                    self.sql_url,
+                    detect_types=sqlite3.PARSE_DECLTYPES,
                 )
                 try:
                     cursor.execute("BEGIN TRANSACTION")
@@ -857,7 +893,7 @@ class Plugin(openpaperwork_core.PluginBase):
                     )
                 finally:
                     cursor.execute("ROLLBACK")
-                    cursor.close()
+                    self.core.call_one("sqlite_close", cursor, optimize=False)
             finally:
                 self.classifiers_cond.notify_all()
 
@@ -1002,4 +1038,4 @@ class Plugin(openpaperwork_core.PluginBase):
             self.core.call_success("doc_add_label_by_url", doc_url, label)
 
     def tests_cleanup(self):
-        self.sql.close()
+        self.core.call_success("sqlite_close", self.sql)
